# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:29
# @Author   : Fangyang
# @Software : PyCharm
from gulf.dolphindb.const import DolType
from gulf.dolphindb.tables.partition.schema import Schema, new_schema


def bond_daily_table_schema() -> Schema:
    """
    ['jj_code', 'trade_date', 'open', 'high', 'low', 'close', 'volume', 'bond_code',
   'bond_name', 'stock_code', 'stock_name', 'bond_scale', 'listing_date',
   'market', 'bond_value', 'trans_stock_value', 'bond_premium',
   'trans_stock_premium', 'duallow']

    :return: None
    """

    bond_daily_cols_type_dict = {
        'jj_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
        'open': DolType.DOUBLE, 'high': DolType.DOUBLE, 'low': DolType.DOUBLE, 'close': DolType.DOUBLE,
        'amount': DolType.DOUBLE, 'volume': DolType.DOUBLE,
        'stock_open': DolType.DOUBLE, 'stock_high': DolType.DOUBLE, 'stock_low': DolType.DOUBLE,
        'stock_close': DolType.DOUBLE, 'stock_amount': DolType.DOUBLE, 'stock_volume': DolType.DOUBLE,
        'bond_code': DolType.SYMBOL, 'bond_name': DolType.SYMBOL,
        'stock_code': DolType.SYMBOL, 'stock_name': DolType.SYMBOL,
        'bond_scale': DolType.DOUBLE, 'listing_date': DolType.NANOTIMESTAMP,
        'market': DolType.SYMBOL, 'trans_stock_value': DolType.DOUBLE,
        # 'bond_value': DolType.DOUBLE,  'bond_premium': DolType.DOUBLE,
        'trans_stock_premium': DolType.DOUBLE, 'duallow': DolType.DOUBLE
    }

    cols_type_dict = dict()
    cols_type_dict.update(bond_daily_cols_type_dict)

    columns_list = [list(bond_daily_cols_type_dict.keys())]

    return new_schema(cols_type_dict, columns_list)


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 8:29
# @Author   : Fangyang
# @Software : PyCharm

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables.partition.schema import industry_moneyflow_daily_table_schema, index_daily_table_schema, \
    hk_hold_table_schema
from gulf.dolphindb.tables.partition.stock_schema import stock_daily_table_schema, stock_moneyflow_daily_table_schema, \
    stock_nfq_daily_table_schema
from gulf.dolphindb.tables.partition.table import PartitionTable


stock_nfq_daily_table = PartitionTable(
    name="stock_nfq_daily_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=stock_nfq_daily_table_schema()
)


stock_daily_table = PartitionTable(
    name="stock_daily_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=stock_daily_table_schema()
)

stock_moneyflow_daily_table = PartitionTable(
    name="stock_moneyflow_daily_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=stock_moneyflow_daily_table_schema()
)

industry_moneyflow_daily_table = PartitionTable(
    name="industry_moneyflow_daily_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=industry_moneyflow_daily_table_schema()
)

index_concept_daily_table = PartitionTable(
    name="index_concept_daily_table",
    db_path=DfsDbPath.index_daily_code,
    schema=index_daily_table_schema()
)

index_industry_daily_table = PartitionTable(
    name="index_industry_daily_table",
    db_path=DfsDbPath.index_daily_code,
    schema=index_daily_table_schema()
)

hk_hold_table = PartitionTable(
    name="hk_hold_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=hk_hold_table_schema()
)

# dfs分区数据库, 建完库以后, 数据表通过python api 写入,
# 根据 dataframe 结构动态定义 schema
stock_moneyflow_hsgt_table = PartitionTable(
    name="moneyflow_hsgt_table",
    db_path=DfsDbPath.stock_daily,
    schema=None
)

stock_fin_table = PartitionTable(
    name="fin_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=None
)

stock_pingji_table = PartitionTable(
    name="pingji_table",
    db_path=DfsDbPath.stock_daily_code,
    schema=None
)

if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 6:52
# @Author   : Fangyang
# @Software : PyCharm

import datetime
from typing import Union, Dict, List

import numpy as np
import pandas as pd
from tqdm import tqdm
from loguru import logger

from gulf.dolphindb.base import Dolphindb
from gulf.dolphindb.const import Engine
from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables import bond_daily_table
from gulf.dolphindb.tables.dimension.bond_tables import BondBasicTable, BondRedeemTable


class BondDB(Dolphindb):
    # 用于 数据库分区 和 确定数据下载范围, TDX 可转债指数从 2020年5月14日 开始
    start_year = 2020
    end_year = 2030

    def __init__(
            self, host: Union[str, None] = None, port: int = 8848,
            username: str = 'admin', password: str = '123456',
            engine: Engine = Engine.TSDB, clear_db: bool = False,
            enable_async: bool = False
    ):
        super().__init__(
            host=host, port=port, username=username, password=password,
            engine=engine, enable_async=enable_async
        )
        self.start_datetime = datetime.datetime(year=self.start_year, month=1, day=1)

        self.init_db(clear=clear_db)

        self.partition_tables = [bond_daily_table, ]
        [self._create_table(table=table) for table in self.partition_tables]

    @property
    def db_path_init_script_dict(self) -> Dict[str, str]:
        return {
            DfsDbPath.bond_daily_code: self.init_daily_code_db_script,
        }

    @property
    def init_daily_code_db_script(self):
        """
        注意! 由于column指定了其他, 不是默认值, [db_code_hash,db_date_range] 修改过顺序
        """

        return f"""
        db_code_hash=database(, HASH, [SYMBOL, 6])
        yearRange=date({self.start_year}.01M + 12*0..{self.end_year - self.start_year})
        db_date_range=database(, RANGE, yearRange)
        db=database('{DfsDbPath.bond_daily_code}', COMPO, [db_code_hash,db_date_range], engine=`{self.engine.value})
        """

    def update_bond_daily_table_by_akshare(self):
        """
        优点: 数据全面, 包含退市债可以达到 48w 数据量, 有纯债价值和纯债溢价率, 但是意义不大在 schema 中注释掉
        缺点:
            1. 下载速度慢, 大概需要5分钟左右,
            2. 下载中途可能因为 sina 限速导致部分数据无法下载成功
        """
        from gulf.akshare.bond import update_bond_daily_res_dict_thread

        res_dict = dict()
        update_bond_daily_res_dict_thread(
            bond_basic_df=self.get_dimension_table_df(BondBasicTable, from_db=True),
            res_dict=res_dict
        )

        self.save_res_dict_to_db_table(partition_table=bond_daily_table, res_dict=res_dict)

    def update_bond_daily_table_by_reader(self, offset: int = 0):
        """
        优点: 读取速度快, 交易中的债几乎不存在缺失问题
        缺点:
            1. 退市债没有数据, 转股溢价率是本地计算, 如果存在复权问题, 可能会存在问题, 需要后续确认,
            2. 没有纯债价值和纯债溢价率

        :param offset: 大于等于0表示将全部数据写入db, -2 表示数据最近2天数据写入db
        """
        from mootdx.reader import Reader

        reader = Reader.factory(market='std', tdxdir='C:/new_tdx')
        bond_basic_df = self.get_dimension_table_df(BondBasicTable, from_db=True)

        res_dict = dict()
        for idx, row in tqdm(bond_basic_df.iterrows()):
            bond_code = row['债券代码']

            # price/10, volume*1000, amount keep
            price_cols = ['open', 'high', 'low', 'close']
            bond_daily_df = reader.daily(symbol=bond_code)
            bond_daily_df = bond_daily_df if offset >= 0 else bond_daily_df.iloc[offset:]

            if bond_daily_df.empty:
                continue
            bond_daily_df.loc[:, price_cols] = bond_daily_df[price_cols] / 10
            bond_daily_df.loc[:, 'volume'] = bond_daily_df['volume'] * 1000

            stock_code = row['正股代码']
            trans_stock_price = row['转股价']

            # amount = price * volume * 100
            stock_daily_df = reader.daily(symbol=stock_code)
            stock_daily_df = stock_daily_df if offset >= 0 else stock_daily_df.iloc[offset:]
            stock_daily_df.columns = [f'stock_{c}' for c in stock_daily_df.columns]

            # 转股溢价率=可转债价格/转股价值-1，转股价值=可转债面值/转股价*正股价。
            dd = pd.concat([bond_daily_df, stock_daily_df], axis=1).dropna()
            dd['trans_stock_value'] = 100 / trans_stock_price * stock_daily_df['stock_close']  # 转股价值
            dd['trans_stock_premium'] = dd['close'] / dd['trans_stock_value'] - 1  # 转股溢价率
            # dd['bond_value'] =  # 纯债价值
            # dd['bond_premium'] =  # 纯债溢价率
            dd['duallow'] = dd['close'] + dd['trans_stock_premium']
            dd['jj_code'] = row['jj_code']
            dd['bond_scale'] = row['发行规模'] if np.isnan(row['剩余规模']) else row['剩余规模']
            dd['listing_date'] = pd.to_datetime(row['上市时间'])
            dd['stock_name'] = row['正股简称']
            dd['bond_name'] = row['债券简称']
            dd['market'] = row['market']
            dd['bond_code'] = bond_code
            dd['stock_code'] = stock_code

            res_dict[bond_code] = dd

        start_datetime, end_datetime = dd.index[0], dd.index[-1]
        logger.success(f"Data from [{start_datetime}] to [{end_datetime}], offset:{offset} trade days")
        df = pd.concat(res_dict.values()).reset_index().rename(columns={'date': 'trade_date'})
        res_dict.clear()
        self.save_res_dict_to_db_table(partition_table=bond_daily_table, res_dict={'df': df})

    def update_dimension_tables(self):
        bond_basic_df = self.get_dimension_table_df(BondBasicTable, from_db=False)
        bond_redeem_df = self.get_dimension_table_df(BondRedeemTable, from_db=False)

    def get_bond_daily_table_df(
            self, table_name: str = "temp_table",
            start_date: str = "2020.01.01",
            end_date: str = "",
            jj_code_list: List[str] = None,
            fields: List[str] = None,
            ctx_by_date_filters: List[str] = None,
            is_indclass_onehot: bool = False,
            end_sql: str = ""
    ) -> pd.DataFrame:
        """
        日常使用习惯方便的sql封装, 增加一些参数可以指定过滤
        1. 连表了StockBasicTable, 增加了行业, 股本, 每股净资产数据
        2. 行业信息可以通过参数指定是否需要onehot
        3. 可扩展的 end_sql

        :param table_name:
        :param start_date, "2005.01.01"
        :param end_date, 格式同上, 默认值为 "", 表示到数据库中最新的数据
        :param jj_code_list: ['SHSE.113635']
        :param fields: List[str], ['bond_scale', "stock_code", "trans_stock_premium"], 全部字段参考bond_daily_table.schema
        :param ctx_by_date_filters:
        :param is_indclass_onehot: 是否行业 onehot encode
        :param end_sql:
        :return: 返回 bond daily Dataframe
        """
        from gulf.dolphindb.tables.dimension.stock_tables import StockBasicTable

        dt = ""
        ctx_by_code_filters = []

        if start_date:
            dt += f"trade_date>={start_date} "
        if end_date:
            dt += f"&& trade_date<={end_date} "

        if dt:
            ctx_by_code_filters.append(dt)

        if jj_code_list:
            ctx_by_code_filters.append(f"jj_code in {jj_code_list}")
        # jj_code_filter = f"jj_code in {jj_code_list}" if jj_code_list else ""

        fields_str = "," + ",".join(fields) if fields else ""

        # filters = ", ".join([dt, jj_code_filter])
        ctx_by_code_filters_str = ", ".join(ctx_by_code_filters)

        if ctx_by_date_filters:
            filter_ctx_by_date = f"""
                    bond_table = select * from bond_table 
                    where {', '.join(ctx_by_date_filters)} 
                    context by trade_date;
                """
        else:
            filter_ctx_by_date = ""

        res = f"""
                use ta;
                db_path = "{bond_daily_table.db_path}";
                db = database(db_path);
                t = loadTable(db, "{bond_daily_table.name}");

                bond_table = select jj_code, bond_name, stock_name, trade_date, stock_code as securityid,
                open, high, low, close, volatile=high-low, volume, rowAvg(close, high, low, open) * volume as amount, 
                (ratios(close)-1)*100 as pct_chg, cumwavg(rowAvg(close, high, low), volume) as vwap,  
                bond_scale, trans_stock_premium, duallow {fields_str}  
                from t 
                where {ctx_by_code_filters_str} 
                context by jj_code;

                {filter_ctx_by_date}

                table6_name = "{StockBasicTable.name}";
                db6_path = "{StockBasicTable.db_path}";
                db6 = database(db6_path);
                table6 = loadTable(db6, table6_name);
                // alpha101 infoData , 没有 cap 总市值, 只有行业信息, 因为cap只与 alpha56 有关

                infoData = select 
                代码 as securityid, 所处行业 as indclass, 
                已流通股份 as stock_cshare, 总股本 as stock_tshare, 每股净资产 as stock_nav  
                from table6 where 所处行业!=NULL;

                {f'infoData = oneHot(infoData, `indclass);' if is_indclass_onehot else ''}

                {table_name} = select * from ej(bond_table, infoData, `securityid) order by jj_code, trade_date;

                {end_sql}

                {table_name};
                """

        return self.session.run(res)


if __name__ == '__main__':
    db = BondDB()

    db.update_dimension_tables()
    # bond_basic_df = db.get_dimension_table_df(BondBasicTable, from_db=True)

    # Note: BondBasicTable 从网上获取的全部转债, 
    # 如果包含退市债, 历史日数据 47 万
    # 如果不包含退市债, 历史日数据 24 万
    # sina 网络的接口有限速, 当前配置可能需要调整
    # db.update_bond_daily_table_by_akshare()

    db.update_bond_daily_table_by_reader(offset=-2)

    # df = db.get_bond_daily_table_df(start_date='2023.10.10', is_indclass_onehot=True)
    print(1)



from enum import Enum
import numpy as np


class DolType(Enum):
    SYMBOL = "SYMBOL"
    NANOTIMESTAMP = "NANOTIMESTAMP"
    DOUBLE = "DOUBLE"
    INT = "INT"


class Engine(Enum):
    TSDB = "TSDB"
    OLAP = "OLAP"


DolType_2_np_dtype_dict = {
    DolType.DOUBLE.value: np.float64,
    DolType.SYMBOL.value: str,
    DolType.INT.value: np.int64,
    DolType.NANOTIMESTAMP.value: 'datetime64[ns]'
}



from random import choice

import requests


def request_session():
    ###########################
    # fangyang add in case ban req
    user_agent_list = [
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/68.0.3440.106 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) Gecko/20100101 Firefox/61.0",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.186 Safari/537.36",
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.62 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.101 Safari/537.36",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)",
        "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10.5; en-US; rv:1.9.2.15) Gecko/20110303 Firefox/3.6.15",
    ]

    # 频繁请求某个网址偶尔会报错请求超时，可采用下面方式降低失败率，设置重试次数为5，会话设置为不维持连接.
    requests.adapters.DEFAULT_RETRIES = 5
    ses = requests.session()
    ses.keep_alive = False

    headers = {
        'User-Agent': choice(user_agent_list),
        'Connection': 'close'
    }

    # proxies = {
    #     "http": "http://127.0.0.1:1080",
    #     "https": "https://127.0.0.1:1080"
    # }
    ses.headers.update(headers)
    ###################

    return ses


# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:09
# @Author   : Fangyang
# @Software : PyCharm

from .stock import StockDB
from .bond import BondDB

if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-


# @Datetime : 2022/12/15 23:37
# @Author   : Fangyang
# @Software : PyCharm

import pandas as pd

from gulf.utils.crawler import request_session


def bond_zh_cov_value_analysis(symbol: str = "123138") -> pd.DataFrame:
    """
    https://data.eastmoney.com/kzz/detail/113527.html
    东方财富网-数据中心-新股数据-可转债数据-价值分析-溢价率分析
    :return: 可转债价值分析
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/get"
    params = {
        "sty": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "st": "date",
        "sr": "1",
        "source": "WEB",
        "type": "RPTA_WEB_KZZ_LS",
        "filter": f'(zcode="{symbol}")',
        "p": "1",
        "ps": "8000",
        "_": "1648629088839",
    }

    ses = request_session()
    r = ses.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "日期",
        "-",
        "-",
        "转股价值",
        "纯债价值",
        "纯债溢价率",
        "转股溢价率",
        "收盘价",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "日期",
            "收盘价",
            "纯债价值",
            "转股价值",
            "纯债溢价率",
            "转股溢价率",
        ]
    ]

    temp_df["日期"] = pd.to_datetime(temp_df["日期"]).dt.date
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"])
    temp_df["纯债价值"] = pd.to_numeric(temp_df["纯债价值"])
    temp_df["转股价值"] = pd.to_numeric(temp_df["转股价值"])
    temp_df["纯债溢价率"] = pd.to_numeric(temp_df["纯债溢价率"])
    temp_df["转股溢价率"] = pd.to_numeric(temp_df["转股溢价率"])

    return temp_df


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 8:29
# @Author   : Fangyang
# @Software : PyCharm

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables.partition.table import PartitionTable
from gulf.dolphindb.tables.partition.bond_schema import bond_daily_table_schema

bond_daily_table = PartitionTable(
    name="bond_daily_table",
    db_path=DfsDbPath.bond_daily_code,
    schema=bond_daily_table_schema()
)

if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:22
# @Author   : Fangyang
# @Software : PyCharm

from functools import cache
from gulf.akshare.stock.migrate_func import stock_zh_a_spot_em
from gulf.utils.code2code import trans_to_juejin_code


@cache
def get_all_stocks_df():
    """
    tushare 的 stock_basic 包含 行业, 地域, 上市日期字段, 能区分退市股票,
    tushare pro 把这个接口的调用积分提高到2000, 导致无法调用，
    换回akshare, 考虑行业和地域信息用途不大从数据表中剔除，上市和退市日期akshare有其他接口调用
    dolphindb 中使用 tushare 接口替换这个方法
    :return:
    """
    df = stock_zh_a_spot_em()

    if "序号" in df.columns:
        df.drop("序号", inplace=True, axis=1)

    df['jj_code'] = df['代码'].apply(trans_to_juejin_code)
    return df


if __name__ == '__main__':
    import time
    # # 这里测试确实cache生效了
    start_time = time.perf_counter()
    a1 = get_all_stocks_df()
    print(f"Time cost:{time.perf_counter() - start_time:.2f}s")
    start_time = time.perf_counter()
    a2 = get_all_stocks_df()
    print(f"Time cost:{time.perf_counter()-start_time:.2f}s")
    start_time = time.perf_counter()
    a3 = get_all_stocks_df()
    print(f"Time cost:{time.perf_counter()-start_time:.2f}s")
    print(1)




from gulf.akshare.bond.all_basic import get_bond_basic_df
from gulf.akshare.bond.history import get_bond_index_daily, get_bond_daily_df, update_bond_daily_res_dict_thread

if __name__ == '__main__':
    get_bond_basic_df()
    get_bond_index_daily()
    get_bond_daily_df()
    update_bond_daily_res_dict_thread()



from typing import Union
from gulf.dolphindb.tables.partition.schema import Schema, crypto_depth_table_schema


class StreamTable:
    def __init__(
            self, name: str, schema: Union[Schema, None],
            length: int = 10000, key: str = "`jj_code`trade_date",
            is_persist: bool = True, cache_size: int = None, pre_cache: int = None
    ):
        """

        :param name:
        :param schema: 默认 crypto_order_book_table_schema(20)， 创建上下20档 order book
        :param length:
        :param key:
        :param is_persist: 是否持久化
        :param cache_size: 默认为10倍流表长度， 流表数据量达到 cache_size 行时持久化
        :param pre_cache: 默认为5倍流表长度， 持久化时，pre_cache 行进行异步方式压缩保存

        若执行enableTableShareAndPersistence时，
        磁盘上已经存在sharedPubTable表的持久化数据，那么系统会加载最新的preCache行记录到内存中。

        对于持久化是否启用异步，需要在持久化数据一致性和性能之间作权衡。当流数据的一致性要求较高时，
        可以使用同步方式，这样可以保证持久化完成以后，数据才会进入发布队列；
        若对实时性要求较高，不希望磁盘IO影响到流数据的实时性，则可启用异步方式。
        只有启用异步方式时，持久化工作线程数persistenceWorkerNum配置项才会起作用。
        若有多个发布表需要持久化，增加persistenceWorkerNum的配置值可以提升异步保存的效率。
        """

        assert length > 0
        assert isinstance(schema, Schema)

        self.name = name
        self.schema = schema
        self.length = length
        self.key = key
        self.is_persist = is_persist
        self.cache_size = cache_size if cache_size else length * 10
        self.pre_cache = pre_cache if pre_cache else length * 5

    def create_table(self):
        """
        https://dolphindb.cn/cn/help/FunctionsandCommands/CommandsReferences/e/enableTableShareAndPersistence.html?highlight=enabletableshareand

        enableTableShareAndPersistence(
        table, tableName, [asynWrite=true], [compress=true], [cacheSize=-1],
        [retentionMinutes=1440], [flushMode=0], [preCache])
        其中的retentionMinutes参数可以控制自动清除过期数据的时间，是一个整数，表示log文件的保留时间（从文件的最后修改时间开始计算），
        单位是分钟。默认值是1440，即一天。
        上面提问中的代码没有设置retentionMinutes的值，就是默认的1440，即一天。在节点正常运行的情况下，会自动清除磁盘上24h前的数据。
        :return:
        """
        s = f"""
            t = keyedStreamTable({self.key}, {self.length}:0,  {self.schema.columns}, [{",".join(self.schema.types)}])
        """
        if self.is_persist:
            persist_s = f"""
            enableTableShareAndPersistence(table=t, tableName=`{self.name}, cacheSize={self.cache_size}, preCache={self.pre_cache})
            """
        else:
            persist_s = f"""
            share t as {self.name}
            """

        script = (
            f"{s}"
            f"{persist_s}"
        )

        return script

    def create_ts_agg_table(
            self,
            table_name: Union[str, None] = None,
            window_size: int = 300,
            step: int = 30,
            metrics: Union[str, None] = None,
            dummy_table: Union[str, None] = None,
            output_table: Union[str, None] = None,
            time_column: str = "`trade_date",
            key_column: str = "`jj_code`trade_date"
    ):
        """
        通过以下方式订阅
        subscribeTable(tableName="Trade", actionName="act_tsaggr", offset=0, handler=append!{tsAggrKline}, msgAsTable=true);

        :param table_name:
        :param window_size:
        :param step:
        :param metrics: <[first(Price), max(Price),min(Price),last(Price),sum(volume)]>
        :param dummy_table:
        :param output_table:
        :param time_column:
        :param key_column:
        :return:
        """

        table_name = table_name if table_name else f"{self.name}_ts_agg"

        return f"""
            {table_name} = createTimeSeriesAggregator(
                name={table_name}, 
                windowSize={window_size}, 
                step={step}, 
                metrics={metrics}, 
                dummyTable={dummy_table}, 
                outputTable={output_table}, 
                timeColumn={time_column}, 
                keyColumn={key_column}
            );
        """

    def clear_persist(self):
        return f"""
            clearTablePersistence({self.name})
        """

    def close_persist(self):
        return f"""
            disableTablePersistence({self.name})
        """

    def get_persist_info(self):
        return f"""
            getPersistenceMeta({self.name})
        """

    @staticmethod
    def get_all_persist_table():
        """
        db.session.run(table.get_all_persist_table())
        返回全部持久化 stream table 的 table_name list
        :return:
        """
        return f"""
        exec filename from files(getConfigure("persistenceDir")+"/") where filename != "persistOffset" 
        """

    @staticmethod
    def cancel_all_subscribe():
        return f"""
            t = getStreamingStat().pubTables
            for(row in t){{
                tableName = row.tableName
                actions = split(substr(row.actions, 1, strlen(row.actions)-2), ",")
                for(action in actions){{
                    unsubscribeTable(tableName=tableName, actionName=action)
                }}
            }}
        """

    def drop_table(self):
        """
        删除内存和磁盘上的表, 需要在没有订阅的时候才能删除, 否则报错
        """
        return f"""
            {self.cancel_all_subscribe()}
            dropStreamTable(`{self.name})
        """

    def update_table(self):
        """
        注意！！这里一定不能换行！！！！
        :return:
        """
        return f"""tableInsert{{{self.name}, }}"""


if __name__ == '__main__':
    import time
    import numpy as np
    import pandas as pd
    from gulf.dolphindb.crypto import CryptoDB

    db = CryptoDB()
    table_name = "btc_table"

    # pub
    t = StreamTable(name=table_name, schema=crypto_depth_table_schema(1))
    db.session.run(t.create_table())
    for i in range(5):
        df = pd.DataFrame(
            [[
                np.datetime64("now"),
                "btc",
                np.random.randn(),
                np.random.randn(),
                np.random.randn(),
                np.random.randn(),
            ]],
            columns=['trade_date', "jj_code", 'ap0', "av0", "bp0", "bv0"]
        )
        time.sleep(1)
        db.session.run(t.update_table(), df)





# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 22:43
# @Author   : Fangyang
# @Software : PyCharm

import threading
import time
import numpy as np
import pandas as pd

from typing import Dict
from datetime import datetime
from tqdm import tqdm
from loguru import logger

from gulf.akshare.const import AKSHARE_CRAWL_CONCURRENT_LIMIT, AKSHARE_CRAWL_STOP_INTERVEL
from gulf.akshare.bond.cov_value_analysis import bond_zh_cov_value_analysis

import akshare as ak

from gulf.akshare.stock.migrate_func import stock_zh_a_hist


def get_bond_daily_df(
        bond_code: str, bond_name: str, bond_scale: float,
        listing_date: datetime,
        stock_code: str, stock_name: str,
        market: str,
        res_dict: Dict[str, pd.DataFrame]
):
    """
    scale = row['发行规模'] if np.isnan(row['剩余规模']) else row['剩余规模']
    :param bond_scale:
    :param res_dict:
    :param bond_code:
    :param bond_name:
    :param listing_date:
    :param stock_code:
    :param stock_name:
    :param market:
    :return:
    """
    # print(f"start deal {bond_code} {bond_name}")

    if listing_date >= pd.Timestamp(datetime.now().date()):
        return

    market_str = "sh" if market == "SHSE" else "sz"
    try:
        # 2. 获取单只债券 ohlcv 全部历史信息
        bond_zh_hs_cov_daily_df = ak.bond_zh_hs_cov_daily(symbol=f"{market_str}{bond_code}")
    except Exception as e:
        logger.warning(f"[bond_zh_hs_cov_daily] {market_str}{bond_code} {bond_name},  retry change market. {e}")
        market_str = "sz" if market == "SHSE" else "sh"
        try:
            bond_zh_hs_cov_daily_df = ak.bond_zh_hs_cov_daily(symbol=f"{market_str}{bond_code}")
            logger.success(f"[bond_zh_hs_cov_daily] {market_str}{bond_code} {bond_name},  retry change market pass")
        except Exception as e1:
            logger.error(
                f"[bond_zh_hs_cov_daily] {market_str}{bond_code} {bond_name},  retry change market fail. {e1}")
            return

    bond_zh_hs_cov_daily_df['bond_code'] = bond_code
    bond_zh_hs_cov_daily_df['bond_name'] = bond_name
    bond_zh_hs_cov_daily_df['stock_code'] = stock_code
    bond_zh_hs_cov_daily_df['stock_name'] = stock_name
    bond_zh_hs_cov_daily_df['bond_scale'] = bond_scale
    bond_zh_hs_cov_daily_df['listing_date'] = listing_date
    bond_zh_hs_cov_daily_df['market'] = market
    # 不加了, 因为存在下修问题, 需要另做处理, 后续因为拿到溢价率了, 所以这里忽略
    # bond_zh_hs_cov_daily_df['trans_stock_price'] = row['转股价']

    bond_zh_hs_cov_daily_df.rename(columns={'date': 'trade_date'}, inplace=True)

    try:
        # 3. 获取单只债券历史 价值 和 溢价率
        bond_hist_df = bond_zh_cov_value_analysis(symbol=bond_code) \
            .rename(
            columns={
                '日期': 'trade_date',
                '纯债价值': 'bond_value',
                '转股价值': 'trans_stock_value',
                '纯债溢价率': 'bond_premium',
                '转股溢价率': 'trans_stock_premium',
            }) \
            .drop(["收盘价", 'bond_value', 'bond_premium'], axis=1)
    except Exception as e:
        logger.error(f"[bond_zh_cov_value_analysis] {bond_code} {bond_name}. {e}")
        return

        # 合并历史 ohclv 与 溢价率 表
    bond_df = bond_zh_hs_cov_daily_df.merge(bond_hist_df, on='trade_date')

    if bond_df.empty:
        return

    bond_df['amount'] = (bond_df['close'] + bond_df['open'] + bond_df['high'] + bond_df['low']) * bond_df['volume'] / 4
    bond_df['trade_date'] = pd.to_datetime(bond_df['trade_date'])
    bond_df['jj_code'] = bond_df['market'] + "." + bond_df['bond_code']
    bond_df['duallow'] = bond_df['close'] + bond_df['trans_stock_premium']

    nfq_df_columns_dict = {
        # trade_date2 只为调试过程中检查数据, 写入数据库时被过滤掉了
        '日期': "trade_date2",
        '开盘': "stock_open", '收盘': "stock_close", '最高': "stock_high", '最低': "stock_low",
        '成交量': "stock_volume", '成交额': "stock_amount",
        # '涨跌幅': "pct_chg",
        # 下面这俩字段在写数据库时被过滤掉了
        # '涨跌额': "pct_amount", '换手率': "turnover",
        # '振幅': 'amp'
    }
    start_date, end_date = bond_df['trade_date'].dt.strftime('%Y%m%d').values[[0, -1]]
    try:
        nfq_df = stock_zh_a_hist(
            symbol=stock_code, period="daily",
            start_date=start_date, end_date=end_date,
            adjust=""
        )
        nfq_df = nfq_df[nfq_df_columns_dict.keys()].rename(columns=nfq_df_columns_dict)
    except Exception as e:
        logger.error(f"[stock_zh_a_hist] {stock_code} {stock_name} may be limited. {e}")
        return

    bond_df = pd.concat([bond_df, nfq_df], axis=1)

    res_dict[bond_code] = bond_df
    return bond_df


def update_bond_daily_res_dict_thread(bond_basic_df: pd.DataFrame, res_dict: Dict[str, pd.DataFrame]):
    start_time = time.perf_counter()
    t_list = []
    for idx, row in tqdm(bond_basic_df.iterrows()):
        bond_code = row['债券代码']
        bond_name = row['债券简称']
        listing_date = pd.to_datetime(row['上市时间'])
        bond_scale = row['发行规模'] if np.isnan(row['剩余规模']) else row['剩余规模']
        stock_code = row['正股代码']
        stock_name = row['正股简称']
        market = row['market']

        # For debug
        # df = get_bond_daily_df(
        #     bond_code, bond_name, bond_scale, listing_date,
        #     stock_code, stock_name, market, res_dict
        # )

        t = threading.Thread(
            target=get_bond_daily_df,
            args=(
                bond_code, bond_name, bond_scale, listing_date,
                stock_code, stock_name, market, res_dict
            )
        )
        t.start()
        t_list.append(t)

        if len(t_list) > AKSHARE_CRAWL_CONCURRENT_LIMIT:
            [t.join() for t in t_list]
            t_list = []
            time.sleep(AKSHARE_CRAWL_STOP_INTERVEL)

    if t_list:
        [t.join() for t in t_list]
    logger.success(
        f"Items: {len(res_dict)}/{len(bond_basic_df)}. "
        f"Threading time cost: {time.perf_counter() - start_time:.2f}s"
    )


def get_bond_index_daily():
    """
    名称	                类型	    描述
    price_dt            object	日期
    price               float64	指数
    amount              float64	剩余规模(亿元)
    volume              float64	成交额(亿元)
    count               int64	数量
    increase_val	    float64	涨跌
    increase_rt	        float64	涨幅
    avg_price	        float64	平均价格(元)
    mid_price	        float64	中位数价格(元)
    mid_convert_value	float64	中位数转股价值
    avg_dblow           float64	平均双底
    avg_premium_rt	    float64	平均溢价率
    mid_premium_rt	    float64	中位数溢价率
    avg_ytm_rt	        float64	平均收益率
    turnover_rt	        float64	换手率
    price_90	        int64	>90
    price_90_100	    int64	90~100
    price_100_110	    int64	100~110
    price_110_120	    int64	110~120
    price_120_130	    int64	120~130
    price_130	        int64	>130
    increase_rt_90	    float64	>90涨幅
    increase_rt_90_100	float64	90~100涨幅
    increase_rt_100_110	float64	100~110涨幅
    increase_rt_110_120	float64	110~120涨幅
    increase_rt_120_130	float64	120~130涨幅
    increase_rt_130	    float64	>130涨幅
    idx_price	        float64	沪深300指数
    idx_increase_rt	    float64	沪深300指数涨幅
    :return:
    """
    bond_cb_index_jsl_df = ak.bond_cb_index_jsl().rename(
        columns={'price_dt': 'trade_date'}
    )
    return bond_cb_index_jsl_df


if __name__ == '__main__':
    from gulf.akshare.bond import get_bond_basic_df

    df1, df2 = get_bond_basic_df(data_delist_status='exclude')
    res_dict = dict()
    update_bond_daily_res_dict_thread(
        bond_basic_df=df1,  # .iloc[:70],
        res_dict=res_dict
    )
    print(1)

    df = get_bond_index_daily()



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:10
# @Author   : Fangyang
# @Software : PyCharm

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables.dimension.bond_tables import BondRedeemTable, BondBasicTable, BondDelistTable, BondIndexTable
from gulf.dolphindb.tables.dimension.stock_tables import StockBasicTable, TradeCalenderTable

from gulf.dolphindb.tables.partition.stock_tables import (
    stock_daily_table, hk_hold_table, stock_fin_table,
    stock_pingji_table, stock_moneyflow_daily_table, industry_moneyflow_daily_table, stock_nfq_daily_table
)

from gulf.dolphindb.tables.partition.bond_tables import bond_daily_table

# 大写驼峰为 维度表, 小写为 分区表
db_table_map = {
    DfsDbPath.trade_calender: [TradeCalenderTable],

    DfsDbPath.stock_basic: [StockBasicTable],
    # DfsDbPath.stock_index: [IndexHS300MembersTable, IndexHS300DailyTable],
    DfsDbPath.stock_daily_code: [
        stock_nfq_daily_table,
        stock_daily_table,
        hk_hold_table,
        stock_fin_table,
        stock_pingji_table,
        stock_moneyflow_daily_table,
        industry_moneyflow_daily_table,
    ],
    DfsDbPath.stock_daily: [],

    DfsDbPath.bond_basic: [BondBasicTable, BondRedeemTable, BondDelistTable],
    DfsDbPath.bond_index: [BondIndexTable],
    DfsDbPath.bond_daily_code: [
        bond_daily_table,
    ]
}

if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/3/5 21:06
# @Author   : Fangyang
# @Software : PyCharm

from tqdm import tqdm
import time
import numpy as np
import pandas as pd
from gulf.dolphindb.crypto import CryptoDB
from gulf.dolphindb.tables.stream.stream_table import StreamTable
from gulf.dolphindb.tables.partition.schema import crypto_depth_table_schema

table_name = "btc_table"

if __name__ == '__main__':
    db = CryptoDB()

    # pub
    t = StreamTable(name=table_name, schema=crypto_depth_table_schema(1))

    try:
        db.session.run(t.create_table())
    except Exception as e:
        print(e)

    for i in tqdm(range(10000)):
        # 注意！column 的顺序要和 stream schema 一致，否则grafana不显示数据
        df = pd.DataFrame(
            [[
                "btc",
                np.datetime64("now"),
                np.random.randn(),
                np.random.randn(),
                np.random.randn(),
                np.random.randn(),
            ]],
            columns=["jj_code", 'trade_date', "bp0", "bv0", 'ap0', "av0"]
        )
        db.session.run(t.update_table(), df)
        print(df)
        # 如果 dolphindb 的 session(enableASYNC=True)，session.run()
        print(f"[Dolphindb] Write stream table {db.session.run(t.get_all_persist_table())}")
        print(f"[Dolphindb] Write stream table {db.session.run(t.get_persist_info())}")
        time.sleep(1)



from typing import Tuple

import numpy as np
import pandas as pd
import requests


def bond_cb_redeem_jsl() -> pd.DataFrame:
    """
    集思录可转债-强赎
    https://www.jisilu.cn/data/cbnew/#redeem
    :return: 集思录可转债-强赎
    :rtype: pandas.DataFrame
    """
    url = "https://www.jisilu.cn/data/cbnew/redeem_list/"
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "5",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.jisilu.cn",
        "Origin": "https://www.jisilu.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.jisilu.cn/data/cbnew/",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="101", "Google Chrome";v="101"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.4951.67 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "___jsl": "LST___t=1653394005966",
    }
    payload = {
        "rp": "50",
    }
    r = requests.post(url, params=params, json=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame([item["cell"] for item in data_json["rows"]])
    temp_df.rename(columns={
        'bond_id': "代码",
        "bond_nm": "名称",
        'price': "现价",
        'stock_id': "正股代码",
        'stock_nm': "正股名称",
        'orig_iss_amt': "规模",
        'curr_iss_amt': "剩余规模",
        'convert_dt': "转股起始日",
        'convert_price': "转股价",
        'redeem_price_ratio': "强赎触发比",
        'real_force_redeem_price': "强赎价",
        'redeem_tc': "强赎条款",
        'sprice': "正股价",
        'redeem_icon': "强赎状态",
        'redeem_count': "强赎天计数",
        'force_redeem_price': "强赎触发价",
    }, inplace=True)

    temp_df = temp_df[
        [
            "代码",
            "名称",
            "现价",
            "正股代码",
            "正股名称",
            "规模",
            "剩余规模",
            "转股起始日",
            "转股价",
            "强赎触发比",
            "强赎触发价",
            "正股价",
            "强赎价",
            "强赎天计数",
            "强赎条款",
            "强赎状态",
        ]
    ]
    temp_df["现价"] = pd.to_numeric(temp_df["现价"])
    temp_df["规模"] = pd.to_numeric(temp_df["规模"])
    temp_df["剩余规模"] = pd.to_numeric(temp_df["剩余规模"])
    temp_df["转股起始日"] = pd.to_datetime(temp_df["转股起始日"]).dt.date
    temp_df["转股价"] = pd.to_numeric(temp_df["转股价"])
    temp_df["强赎触发比"] = pd.to_numeric(temp_df["强赎触发比"].str.strip("%"))
    temp_df["强赎触发价"] = pd.to_numeric(temp_df["强赎触发价"])
    temp_df["正股价"] = pd.to_numeric(temp_df["正股价"])
    temp_df["强赎价"] = pd.to_numeric(temp_df["强赎价"], errors="coerce")
    temp_df["强赎天计数"] = temp_df["强赎天计数"].replace(
        r"^.*?(\d{1,2}\/\d{1,2} \| \d{1,2}).*?$", r"\1", regex=True
    )
    temp_df["强赎状态"] = temp_df["强赎状态"].map(
        {"R": "已公告强赎", "O": "公告要强赎", "G": "公告不强赎", "B": "已满足强赎条件", "": ""}
    )
    return temp_df


def get_bond_cb_redeem_jsl_df() -> Tuple[pd.DataFrame, pd.DataFrame]:
    bond_cb_redeem_jsl_df = bond_cb_redeem_jsl().drop(
        labels=['正股价', '转股价'],
        axis=1
    )
    bond_cb_redeem_jsl_df['强赎天计数'] = bond_cb_redeem_jsl_df['强赎天计数'].replace(
        to_replace=r'\<.*\>',
        value="",
        regex=True
    )
    bond_cb_redeem_jsl_df['不安全天数'] = (
        bond_cb_redeem_jsl_df['强赎天计数']
        .apply(lambda x: int(x.split('/')[0]) if '/' in x else 30)
    )

    bond_cb_redeem_jsl_df['强赎状态'] = bond_cb_redeem_jsl_df['强赎状态'].replace(
        to_replace=r'^\s*$',
        value="不强赎",
        regex=True
    )
    bond_cb_redeem_jsl_df.rename(columns={"代码": "债券代码"}, inplace=True)
    redeem_jsl_df = bond_cb_redeem_jsl_df[~bond_cb_redeem_jsl_df['强赎状态'].str.contains('不强赎')].copy()
    return bond_cb_redeem_jsl_df, redeem_jsl_df


if __name__ == '__main__':
    df, r_df = get_bond_cb_redeem_jsl_df()
    print(1)



from typing import Union

from gulf.dolphindb.tables.partition.schema import Schema


class PartitionTable:
    def __init__(self, name: str, db_path: str, schema: Union[Schema, None]):
        self.name = name
        self.db_path = db_path
        self.schema = schema



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 22:37
# @Author   : Fangyang
# @Software : PyCharm
from functools import cache
from typing import Tuple

import akshare as ak
import pandas as pd

from gulf.akshare.bond.redeem import get_bond_cb_redeem_jsl_df
from gulf.utils.code2code import MARKET_SH, MARKET_SZ, JUEJIN_MARKET_MAP, trans_to_juejin_code


@cache
def get_bond_basic_df(data_delist_status: str = "include") -> Tuple[pd.DataFrame, pd.DataFrame]:
    """
    返回 (bond_zh_cov_df, redeem_df）

    data_delist_status: "include" / "exclude" , 包含退市债 / 不包含退市债

    全部转债信息(包含退市的)
    Index(['债券代码', '债券简称', '申购日期', '申购代码', '申购上限', '正股代码', '正股简称', '正股价', '转股价',
            '转股价值', '债现价', '转股溢价率', '原股东配售-股权登记日', '原股东配售-每股配售额', '发行规模', '中签号发布日',
            '中签率', '上市时间'],
            dtype='object')

    """

    # 1. 获取全部债券信息, 包含退市债券
    bond_zh_cov_df = ak.bond_zh_cov()

    bond_zh_cov_df.loc[bond_zh_cov_df['债券代码'].str.contains("^12\d{4}"), "market"] = JUEJIN_MARKET_MAP[MARKET_SZ]
    bond_zh_cov_df.loc[bond_zh_cov_df['债券代码'].str.contains("^11\d{4}"), "market"] = JUEJIN_MARKET_MAP[MARKET_SH]

    bond_cb_redeem_jsl_df, redeem_df = get_bond_cb_redeem_jsl_df()

    if data_delist_status == "include":
        bond_zh_cov_df = bond_zh_cov_df.merge(bond_cb_redeem_jsl_df, on=["债券代码", '正股代码'], how="outer")
    elif data_delist_status == "exclude":
        bond_zh_cov_df = bond_zh_cov_df.merge(bond_cb_redeem_jsl_df, on=["债券代码", '正股代码'])
        bond_zh_cov_df = bond_zh_cov_df[bond_zh_cov_df['强赎状态'].str.contains('不强赎')]

    bond_zh_cov_df.drop(
        labels=['名称', "正股名称", "申购日期", "申购代码", "申购上限", "原股东配售-股权登记日",
                "原股东配售-每股配售额", "中签号发布日"],
        axis=1, inplace=True
    )

    # 去除 bond_cb_redeem_jsl_df 中的 EB 可交债
    bond_zh_cov_df = bond_zh_cov_df[~bond_zh_cov_df['债券简称'].isnull()]
    # 去除未上市债券
    bond_zh_cov_df = bond_zh_cov_df[~bond_zh_cov_df['上市时间'].isnull()]

    bond_zh_cov_df.reset_index(drop=True, inplace=True)

    bond_zh_cov_df['jj_code'] = bond_zh_cov_df['market'] + "." + bond_zh_cov_df['债券代码']
    redeem_df['jj_code'] = redeem_df['债券代码'].apply(trans_to_juejin_code)

    return bond_zh_cov_df, redeem_df


if __name__ == '__main__':
    import time

    start_time = time.perf_counter()
    df1, df2 = get_bond_basic_df()
    print(f"Time cost: {time.perf_counter() - start_time}s")

    start_time = time.perf_counter()
    df1, df2 = get_bond_basic_df()
    print(f"Time cost: {time.perf_counter() - start_time}s")

    start_time = time.perf_counter()
    df1, df2 = get_bond_basic_df()
    print(f"Time cost: {time.perf_counter() - start_time}s")

    print(1)



import socket


def get_lan_ip() -> str:
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        s.connect(("8.8.8.8", 80))
        return s.getsockname()[0]


if __name__ == '__main__':
    ip = get_lan_ip()
    print(ip)



import time

import dolphindb as ddb
import dolphindb.settings as keys
import pandas as pd

from typing import Dict, Type, Union
from loguru import logger

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.const import DolType_2_np_dtype_dict, Engine
from gulf.dolphindb.tables.dimension.table import DimensionTable
from gulf.dolphindb.tables.partition.table import PartitionTable
from gulf.utils.network import get_lan_ip


class Dolphindb:

    def __init__(
            self, host: str = None, port: int = 8848,
            username: str = 'admin', password: str = '123456',
            engine: Engine = Engine.TSDB, enable_async: bool = False
    ):
        """
        :param host:
        :param port:
        :param username:
        :param password:
        """
        self.host = host if host else get_lan_ip()  # 这里注意clash不能使用TUN mode, 否则获取的ip地址不是局域网ip
        self.port = port
        logger.info(f"Connect to database {self.host}:{self.port}")

        self.username = username
        self.password = password

        # enableASYNC=True , 默认值是 False
        # stream table 异步持久化需要这里设置为True, 否则报错
        # 但是 异步时 session.run() 会没有返回结果
        self.session = ddb.session(enableASYNC=enable_async)
        self.engine = engine

        self.session.connect(host=self.host, port=self.port, userid=self.username, password=self.password)

        # 创建连接池（用于数据写入）
        self.pool = ddb.DBConnectionPool(self.host, self.port, 1, self.username, self.password)

    @property
    def db_path_init_script_dict(self) -> Dict[str, str]:
        raise NotImplementedError

    def init_db(self, clear: bool = False):
        """
        :param clear: 默认为 False, 如果为 True, 强制清空数据库, 然后重新建立分区数据库
        """
        if clear:
            for db_path in self.db_path_init_script_dict.keys():
                if self.session.existsDatabase(db_path):
                    self.session.dropDatabase(db_path)
                    logger.info(f"Drop database: {db_path}")

        for db_path, script in self.db_path_init_script_dict.items():
            if not self.session.existsDatabase(db_path):
                self.session.run(script)

    def _create_table(
            self, table: PartitionTable,
            partition_sort_columns: str = "`jj_code`trade_date"
    ):
        """

        :param table:
        :param partition_sort_columns: 排序要把 时间写在最后, TSDB
        :return:
        """
        columns = table.schema.columns
        types = table.schema.types
        db_path = table.db_path
        table_name = table.name

        tsdb_args = f", sortColumns={partition_sort_columns}, keepDuplicates=FIRST" if self.engine == Engine.TSDB else ""
        create_table_script = f"""
                db=database('{db_path}')
                columns={columns}
                types={types}
                {table_name}=db.createPartitionedTable(table=table(100000:0,columns,types), tableName=`{table_name}, partitionColumns={partition_sort_columns}{tsdb_args})
                """  # 10_0000

        if not self.session.existsTable(dbUrl=db_path, tableName=table_name):
            self.session.run(create_table_script)
            logger.info(f"Create table db_path:{db_path}, table_name: {table_name}")
        else:
            logger.info(f"Table exist. No create, db_path:{db_path}, table_name: {table_name}")

    def get_dimension_table_df(
            self, dim_table_cls: Type[DimensionTable], from_db: bool = True
    ) -> Union[pd.DataFrame, None]:
        if from_db:
            try:
                df = self.session.loadTable(dim_table_cls.name, dim_table_cls.db_path).toDF()
                return df
            except Exception:
                logger.exception(f'Dimension table: {dim_table_cls.name}. DB path: {dim_table_cls.db_path}')
                return None
        else:
            table = dim_table_cls()
            self.save_dimension_table(dim_table=table)
            return table.df

    def save_dimension_table(self, dim_table: DimensionTable):
        """
        维度表, 用于存储元信息, 简短的表
        :param dim_table:
        :return:
        """

        db_path = dim_table.db_path
        table_name = dim_table.name

        if dim_table.df.empty:
            logger.error(f"Fail to save dimension table, {table_name} to {db_path}, table.df is empty")
            return

        if self.session.existsTable(dbUrl=db_path, tableName=table_name):
            self.session.dropTable(dbPath=db_path, tableName=table_name)

        db = self.session.database(
            dbName=dim_table.db_name,
            dbPath=db_path,
            partitionType=keys.VALUE,
            partitions=[1, 2, 3],
            engine=str(self.engine.value)
        )

        table = self.session.table(data=dim_table.df)

        # 创建维度表, 不用 createPartitionedTable
        db.createTable(
            table=table, tableName=table_name, sortColumns=dim_table.sort_columns
        )

        re = self.session.loadTable(tableName=table_name, dbPath=db_path)
        re.append(table=table)
        logger.success(f"Save dimension table, {table_name} to {db_path}")

    def save_res_dict_to_db_table(
            self, partition_table: PartitionTable, res_dict: Dict, partition_col: str = 'jj_code'
    ):
        db_path = partition_table.db_path
        table_name = partition_table.name
        columns = partition_table.schema.columns
        types = partition_table.schema.types

        appender = ddb.PartitionedTableAppender(
            dbPath=db_path,
            tableName=table_name,
            partitionColName=partition_col,
            dbConnectionPool=self.pool
        )

        if not res_dict:
            return

        res_df = pd.concat(res_dict.values())
        res_df = res_df[columns]  # TODO 注意 !!! append 的 df columns 顺序必须和建表时的顺序一致

        # 取到的 volume 列数据类型不一致, 有int和float混合的问题, 导致写数据库的时候报错,
        # 这里做类型一致性检查和修正, 所以datetime类型在修正过程中直接转换了,省去下面这行
        # table_df['trade_date'] = pd.to_datetime(table_df['trade_date'])
        for col, dtype, dol_type in zip(columns, res_df.dtypes, types):
            if DolType_2_np_dtype_dict[dol_type] != dtype:
                res_df[col] = res_df[col].astype(DolType_2_np_dtype_dict[dol_type])

        start_time = time.perf_counter()
        appender.append(res_df)  # TODO 注意 !!! append 的 df columns 顺序必须和建表时的顺序一致
        logger.success(f"Write len:{len(res_df)} data to db, cost:{time.perf_counter() - start_time:.2f}s")

        res_dict.clear()

    def get_tables(self, db_path: str):
        return self.session.run(
            f"""
            db = database('{db_path}');
            getTables(db);
        """
        )

    def get_all_dfs_tables(self):
        r_dict = {}
        for dfs in self.db_path_init_script_dict.keys():
            r_dict[dfs] = self.get_tables(dfs)
        return r_dict

    def get_table_meta(self, db_path: str, table_name: str):
        return self.session.run(
            f"getTabletsMeta('/{db_path.split('://')[1]}/%', `{table_name}, true)"
        )

    def get_all_dfs_tables_meta(self):
        df_list = []
        for dfs, table_names in self.get_all_dfs_tables().items():
            if len(table_names) != 0:
                for table_name in table_names:
                    df_list.append(self.get_table_meta(db_path=dfs, table_name=table_name))
        return pd.concat(df_list)

    def delete_dfs_data_before_trade_date(self, db_path: DfsDbPath, table_name: str, trade_date: str):
        return self.session.run(
            f"""
                db = database('{db_path}');
                t = loadTable(db, `{table_name});
                delete from t where trade_date < {trade_date};
            """
        )


if __name__ == '__main__':
    db = Dolphindb(
        # host='192.168.0.104', port=8848,
        # username='admin', password='123456',
        # engine=Engine.TSDB,
        # clear_db=True
    )

    # df = db.get_all_dfs_tables_meta()
    # db.session.dropDatabase(BondBasicTable.db_path)
    # db.session.dropTable(db.stock_daily_code_db_path, "stock_daily_table")

    # 清空数据库数据
    # db.init_db(clear=True)

    # clear db
    # db.session.dropDatabase(dbPath=db.stock_daily_code_db_path)
    # db.session.dropTable(dbPath=db.stock_daily_code_db_path, tableName=table_name)
    # db.session.dropTable(dbPath=db.stock_daily_db_path, tableName="moneyflow_hsgt_table")

    # t = db.session.loadTable(tableName="moneyflow_hsgt_table", dbPath=db.stock_daily_db_path)
    # r1 = t.select("*").where("trade_date<=2022.09.01").sort(bys="trade_date").toDF()
    # r1.fillna(value=0, inplace=True)
    # r1['upper'], r1['mid'], r1['lower'] = adbband(r1['north_money'], length=15)
    # r1['entry'] = r1['north_money'] > r1['upper']
    # r1['exit1'] = r1['north_money'] < r1['mid']
    # r1['exit2'] = r1['north_money'] < r1['lower']
    # entry = r1.iloc[-1, :]['entry']

    # t = db.session.loadTable(tableName="stock_daily_table", dbPath=db.stock_daily_code_db_path)
    # r1 = t.select("*").where("jj_code='SHSE.600053'").toDF()  # .ols("high", ['low'])
    # r2 = t.select(['jj_code', "circ_mv"]).where("trade_date=2022.09.01").toDF()  # .ols("high", ['low'])
    # r = t.select(['trade_date', 'jj_code', 'low', 'high']).where("jj_code='SHSE.600053'").toDF()  # .ols("high", ['low'])
    # print(r1)

    # olss = t.select(['trade_date', 'jj_code', 'low', 'high']).where("jj_code='SHSE.600053'").ols("high", 'low')
    # df = db.trade_calender

    # t = db.session.loadTable(tableName="hk_hold_table", dbPath=db.stock_daily_code_db_path)
    # r3 = t.select("top 200 *")\
    #     .where("trade_date=2022.09.01, exchange!=`HK")\
    #     .sort(bys="ratio", ascending=False).toDF()  # .ols("high", ['low'])

    # r4 = t.select("top 200 *") \
    #     .where("jj_code='SZSE.002460', exchange!=`HK") \
    #     .sort(bys="ratio", ascending=False).toDF()  # .ols("high", ['low'])

    print(1)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 6:37
# @Author   : Fangyang
# @Software : PyCharm

import datetime
from typing import Union, Dict, List

import pandas as pd
from tqdm import tqdm

from gulf.dolphindb.base import Dolphindb
from gulf.dolphindb.const import Engine
from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables import TradeCalenderTable, StockBasicTable
from gulf.dolphindb.tables.partition.stock_tables import stock_nfq_daily_table


class StockDB(Dolphindb):
    # 从2000年开始, 有两个原因:
    # 1. 会计规则从1999年开始形成
    # 2. 样本数足够多
    # 从2005年开始, 沪深300指数从2005年1月4日起开始有完整数据
    start_year = 2005  # 用于 数据库分区 和 确定数据下载范围
    end_year = 2030

    def __init__(
            self, host: Union[str, None] = None, port: int = 8848,
            username: str = 'admin', password: str = '123456',
            engine: Engine = Engine.TSDB, clear_db: bool = False,
            enable_async: bool = False
    ):
        super().__init__(
            host=host, port=port, username=username, password=password,
            engine=engine, enable_async=enable_async
        )
        self.start_datetime = datetime.datetime(year=self.start_year, month=1, day=1)

        self.init_db(clear=clear_db)

        self.partition_tables = [stock_nfq_daily_table, ]
        [self._create_table(table=table) for table in self.partition_tables]

    @property
    def db_path_init_script_dict(self) -> Dict[str, str]:
        return {
            DfsDbPath.stock_daily_code: self.init_daily_code_db_script,
            DfsDbPath.stock_daily: self.init_daily_db_script,
        }

    @property
    def init_daily_code_db_script(self):
        return f"""
        db_code_hash=database(, HASH, [SYMBOL, 40])
        yearRange=date({self.start_year}.01M + 12*0..{self.end_year - self.start_year})
        db_date_range=database(, RANGE, yearRange)
        db=database('{DfsDbPath.stock_daily_code}', COMPO, [db_code_hash,db_date_range], engine=`{self.engine.value})
        """

    @property
    def init_daily_db_script(self):
        """
        只有日期 没有 股票代码
        """
        return f"""
        yearRange=date({self.start_year}.01M + 12*0..{self.end_year - self.start_year})
        db=database('{DfsDbPath.stock_daily}', RANGE, yearRange, engine=`{self.engine.value})
        """

    def update_stock_nfq_daily_table_by_reader(self, offset: int = 0):
        """
        读取本地数据, 写入db, mootdx reader 目前代码不涵盖 bj 路径, 故没有北交所数据
        :param offset: 大于等于0表示将全部数据写入db, -2 表示数据最近2天数据写入db
        :return:
        """
        from mootdx.reader import Reader

        reader = Reader.factory(market='std', tdxdir='C:/new_tdx')
        stock_basic_df = self.get_dimension_table_df(StockBasicTable, from_db=True)

        res_dict = dict()
        for idx, row in tqdm(stock_basic_df.iterrows()):
            stock_code = row['代码']

            stock_daily_df = reader.daily(symbol=stock_code)
            stock_daily_df = stock_daily_df if offset >= 0 else stock_daily_df.iloc[offset:]
            if stock_daily_df.empty:
                continue

            stock_daily_df['jj_code'] = row['jj_code']
            stock_daily_df['name'] = row['名称']
            stock_daily_df['industry'] = row['所处行业']

            res_dict[stock_code] = stock_daily_df

        df = pd.concat(res_dict.values()).reset_index(names=['trade_date'])
        res_dict.clear()
        self.save_res_dict_to_db_table(partition_table=stock_nfq_daily_table, res_dict={'df': df})

    def update_dimension_tables(self):
        trade_calender = self.get_dimension_table_df(TradeCalenderTable, from_db=False)
        stock_basic_df = self.get_dimension_table_df(StockBasicTable, from_db=False)

    def get_stock_daily_table_df(
            self, table_name: str = "temp_table",
            start_date: str = "2005.01.01",
            end_date: str = "",
            where_filter: str = "",
            market: str = "S",
            is_indclass_onehot: bool = False,
            end_sql: str = ""
    ) -> pd.DataFrame:
        """

        :param table_name:
        :param start_date:
        :param end_date:
        :param where_filter:
        :param market: 默认值为 'S' 表示排除北交所, 'SH' 表示沪市, 'SZ'表示深市, 'B' 表示北交所, '' 表示全部,
        :param is_indclass_onehot: 是否行业 onehot encode
        :param end_sql:
        :return:
        """

        dt = ""
        if start_date:
            dt += f"&& trade_date>={start_date} "
        if end_date:
            dt += f"&& trade_date<={end_date} "

        where_filter = "," + where_filter if where_filter else ""

        res = f"""
            use ta;
            db_path = "{stock_nfq_daily_table.db_path}";
            db = database(db_path);
            t = loadTable(db, "{stock_nfq_daily_table.name}");

            def get_raw_data_table(market='SH'){{
                return  select *, cumwavg(rowAvg(close, high, low), volume) as vwap  
                from t 
                where jj_code like concat(market, '%'),  
                jj_code not like '%.68%', jj_code not like '%.30%', name not like '%ST%'  
                {dt} {where_filter}  
                context by jj_code;
            }}

            {table_name} = get_raw_data_table(market='{market}');  // 排除北交所
            {f'{table_name} = oneHot({table_name}, `industry);' if is_indclass_onehot else ''}
            
            {end_sql}
            {table_name};
            """

        return self.session.run(res)


if __name__ == '__main__':
    db = StockDB()
    # db.update_dimension_tables()
    # db.update_stock_nfq_daily_table_by_reader(offset=-1)

    df0 = db.get_stock_daily_table_df(start_date='2023.10.10', is_indclass_onehot=True)
    df1 = db.get_stock_daily_table_df(start_date='2023.10.10', market='S')
    df2 = db.get_stock_daily_table_df(start_date='2023.10.10', market='')
    df3 = db.get_stock_daily_table_df(start_date='2023.10.10', market='B')
    df4 = db.get_stock_daily_table_df(start_date='2023.10.10', market='SH')
    df5 = db.get_stock_daily_table_df(start_date='2023.10.10', market='SZ')
    print(1)


# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:38
# @Author   : Fangyang
# @Software : PyCharm


import pandas as pd


class DimensionTable:
    name: str = ""
    db_path: str = ""
    db_name: str = ""
    sort_columns: str = ""
    df: pd.DataFrame = None


if __name__ == '__main__':
    pass



from typing import Dict, List

from gulf.dolphindb.const import DolType


class Schema:
    def __init__(self, columns_list, columns, types):
        self.columns_list = columns_list
        self.columns = columns
        self.types = types


def new_schema(
        cols_type_dict: Dict[str, DolType],
        columns_list: List[List[str]]
):
    columns = []
    types = []
    for c, t in cols_type_dict.items():
        columns.append(c)
        types.append(t.value)

    return Schema(
        columns_list=columns_list,
        columns=columns,
        types=types
    )


def index_daily_table_schema() -> Schema:
    cols_type_dict = {'jj_code': DolType.SYMBOL, }

    # tushare daily 数据不全, 这里使用akshare数据
    daily_cols_type_dict = {
        'trade_date': DolType.NANOTIMESTAMP,
        'open': DolType.DOUBLE, 'high': DolType.DOUBLE,
        'low': DolType.DOUBLE, 'close': DolType.DOUBLE,
        'pct_chg': DolType.DOUBLE, 'pct_amount': DolType.DOUBLE,
        'vol': DolType.DOUBLE, 'amount': DolType.DOUBLE,
        'hl_chg': DolType.DOUBLE, 'turnover_rate': DolType.DOUBLE,
    }

    cols_type_dict.update(daily_cols_type_dict)
    columns_list = [list(daily_cols_type_dict.keys())]

    return new_schema(cols_type_dict, columns_list)


def industry_moneyflow_daily_table_schema() -> Schema:
    cols_type_dict = {}

    # tushare daily 数据不全, 这里使用akshare数据
    daily_cols_type_dict = {
        'in_amount_main': DolType.DOUBLE, 'in_ratio_main': DolType.DOUBLE,
        'in_amount_large': DolType.DOUBLE, 'in_ratio_large': DolType.DOUBLE,
        'in_amount_big': DolType.DOUBLE, 'in_ratio_big': DolType.DOUBLE,
        'in_amount_mid': DolType.DOUBLE, 'in_ratio_mid': DolType.DOUBLE,
        'in_amount_small': DolType.DOUBLE, 'in_ratio_small': DolType.DOUBLE,
        'jj_code': DolType.SYMBOL,
        'trade_date': DolType.NANOTIMESTAMP,
    }

    cols_type_dict.update(daily_cols_type_dict)

    columns_list = [
        list(daily_cols_type_dict.keys()),
    ]

    return new_schema(cols_type_dict, columns_list)


def hk_hold_table_schema() -> Schema:
    """
    ['jj_code', 'ts_code', 'trade_date', 'name', 'vol', 'ratio', 'exchange']

    :return: None
    """

    hk_hold_cols_type_dict = {
        # 'code',
        'ts_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
        'name': DolType.SYMBOL, 'vol': DolType.DOUBLE,
        'ratio': DolType.DOUBLE, 'exchange': DolType.SYMBOL
    }

    cols_type_dict = {'jj_code': DolType.SYMBOL, }
    cols_type_dict.update(hk_hold_cols_type_dict)

    columns_list = [list(hk_hold_cols_type_dict.keys())]

    return new_schema(cols_type_dict, columns_list)


def crypto_depth_table_schema(ask_bid_nums: int = 20) -> Schema:
    """
    ['jj_code', 'trade_date', ...]

    :return: Schema
    """
    bid_price_col = {f"bp{i}": DolType.DOUBLE for i in range(ask_bid_nums)}
    bid_volume_col = {f"bv{i}": DolType.DOUBLE for i in range(ask_bid_nums)}
    ask_price_col = {f"ap{i}": DolType.DOUBLE for i in range(ask_bid_nums)}
    ask_volume_col = {f"av{i}": DolType.DOUBLE for i in range(ask_bid_nums)}

    crypto_order_book_cols_type_dict = {
        'jj_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
    }

    cols_type_dict = dict()
    [cols_type_dict.update(e)
     for e in [crypto_order_book_cols_type_dict, bid_price_col, bid_volume_col, ask_price_col, ask_volume_col]
     ]
    columns_list = [list(crypto_order_book_cols_type_dict.keys())]

    return new_schema(cols_type_dict, columns_list)


def crypto_aggtrade_table_schema() -> Schema:
    """
    ['jj_code', 'trade_date', ...]

    :return: Schema
    """
    crypto_order_book_cols_type_dict = {
        'jj_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
        'p': DolType.DOUBLE, 'q': DolType.DOUBLE,
        'trade_timestamp': DolType.NANOTIMESTAMP,
        'maker': DolType.INT
    }

    cols_type_dict = dict()
    cols_type_dict.update(crypto_order_book_cols_type_dict)
    columns_list = [list(crypto_order_book_cols_type_dict.keys())]

    return new_schema(cols_type_dict, columns_list)


if __name__ == '__main__':
    s = crypto_depth_table_schema()
    s1 = crypto_aggtrade_table_schema()
    print(1)



import json
import threading

from typing import Any, Dict, List

import pandas as pd
from datetime import datetime

from gulf.utils.code2code import trans_to_juejin_code
from gulf.utils.crawler import request_session


def request_data(
        url: str, num: int, params: Dict[str, Any],
        # headers: Dict[str, str],
        columns: List[str],
        res_dict: Dict[str, pd.DataFrame]
) -> dict[str, pd.DataFrame]:
    ses = request_session()
    r = ses.get(url=url, params=params)
    # r = requests.get(url, params=params, headers=headers, timeout=5)

    r.encoding = r.apparent_encoding

    txt = r.text
    if "[]" in txt:
        print("页面已经到头")
        res_dict['end'] = pd.DataFrame()
        return res_dict

    txt = json.loads(txt)
    rate_name_list = txt['data']

    res_dict[f"{num}"] = pd.DataFrame.from_records(rate_name_list)[columns]

    return res_dict


def get_stock_analyst_rating(
        start: str = "2005-01-01",
        end: str = datetime.now().date().strftime("%Y-%m-%d")
):
    """
     点击翻页1就能看到url
    "https://data.eastmoney.com/report/stock.jshtml"
    "https://www.bilibili.com/video/BV1xT411c775/?vd_source=ac309ed33e549fafe9293cd08a468cd4"
    :return:
    """

    num = 1
    all_data = pd.DataFrame()
    url = "https://reportapi.eastmoney.com/report/list"

    columns = [
        'title', 'stockName', 'stockCode', 'orgSName', 'publishDate',
        # 'orgCode', 'orgName', 'infoCode','column',
        'predictNextTwoYearEps', 'predictNextTwoYearPe',
        'predictNextYearEps', 'predictNextYearPe',
        'predictThisYearEps', 'predictThisYearPe',
        # 'predictLastYearEps', 'predictLastYearPe',
        # 'actualLastTwoYearEps', 'actualLastYearEps',
        # 'industryCode', 'industryName', 'emIndustryCode',
        'indvInduCode', 'indvInduName',
        # 'emRatingCode', 'emRatingValue', 'lastEmRatingCode', 'lastEmRatingValue',
        'emRatingName', 'lastEmRatingName',
        # 'ratingChange', 'reportType', 'indvIsNew', 'author',
        'researcher',
        # 'newListingDate', 'newPurchaseDate', 'newIssuePrice', 'newPeIssueA',
        'indvAimPriceT', 'indvAimPriceL',
        # 'attachType', 'attachSize', 'attachPages', 'encodeUrl',
        'sRatingName',
        # 'sRatingCode',
        'market', 'authorID', 'count',
        # 'orgType'
    ]

    res_dict = dict()

    t_list = []
    while True:
        print(f"[{num}]" + "=" * 100)
        params = {
            "industryCode": "*",
            "pageSize": "100",
            "industry": "*",
            "rating": "",
            "ratingChange": "",
            "beginTime": start,
            "endTime": end,
            "pageNo": f"{num}",
            "fields": "",
            "qType": "0",
            "orgCode": "",
            "code": "*",
            "rcode": "",
            "p": f"{num}",
            "pageNum": f"{num}",
            "pageNumber": f"{num}",
            "_": "1660628644570"
        }

        # request_data(url, num, params, columns, res_dict)
        t = threading.Thread(target=request_data, args=(url, num, params, columns, res_dict))
        t.start()
        t_list.append(t)

        num += 1
        if num % 100 == 0:
            [t.join() for t in t_list]
            t_list = []
            if "end" in res_dict:
                break

    del res_dict['end']

    all_data = pd.concat(list(res_dict.values())) \
        .rename(columns={'publishDate': "trade_date"}) \
        .sort_values(by="trade_date", ascending=False).reset_index(drop=True)

    all_data["jj_code"] = all_data['stockCode'].apply(trans_to_juejin_code)
    all_data['authorID'] = all_data['authorID'].apply(lambda x: str(x))

    return all_data


if __name__ == '__main__':
    import time

    start_time = time.perf_counter()
    df:pd.DataFrame = get_stock_analyst_rating()
    print(f"Time cost: {time.perf_counter() - start_time:.2f}s")
    df.to_pickle("pingji.pkl")

    df = pd.read_pickle('pingji.pkl')\
        .rename(columns={'publishDate': "trade_date"})\
        .sort_values(by="trade_date", ascending=False)\
        .reset_index(drop=True)
    df["jj_code"] = df['stockCode'].apply(trans_to_juejin_code)
    df['authorID'] = df['authorID'].apply(lambda x: str(x))
    print(1)



import pandas as pd
from akshare.stock_feature.stock_hist_em import code_id_map_em

from gulf.utils.crawler import request_session


def stock_zh_a_hist(
        symbol: str = "000001",
        period: str = "daily",
        start_date: str = "19700101",
        end_date: str = "20500101",
        adjust: str = "",
) -> pd.DataFrame:
    """
    东方财富网-行情首页-沪深京 A 股-每日行情
    http://quote.eastmoney.com/concept/sh603777.html?from=classic
    :param symbol: 股票代码
    :type symbol: str
    :param period: choice of {'daily', 'weekly', 'monthly'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :param adjust: choice of {"qfq": "前复权", "hfq": "后复权", "": "不复权"}
    :type adjust: str
    :return: 每日行情
    :rtype: pandas.DataFrame
    """
    code_id_dict = code_id_map_em()
    adjust_dict = {"qfq": "1", "hfq": "2", "": "0"}
    period_dict = {"daily": "101", "weekly": "102", "monthly": "103"}
    url = "http://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f116",
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "klt": period_dict[period],
        "fqt": adjust_dict[adjust],
        "secid": f"{code_id_dict[symbol]}.{symbol}",
        "beg": start_date,
        "end": end_date,
        "_": "1623766962675",
    }

    ses = request_session()
    r = ses.get(url, params=params)
    data_json = r.json()

    if not (data_json["data"] and data_json["data"]["klines"]):
        return pd.DataFrame()

    temp_df = pd.DataFrame(
        [item.split(",") for item in data_json["data"]["klines"]]
    )
    temp_df.columns = [
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "振幅",
        "涨跌幅",
        "涨跌额",
        "换手率",
    ]
    temp_df.index = pd.to_datetime(temp_df["日期"])
    temp_df.reset_index(inplace=True, drop=True)

    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"])
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"])
    temp_df["最高"] = pd.to_numeric(temp_df["最高"])
    temp_df["最低"] = pd.to_numeric(temp_df["最低"])
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"])
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"])
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"])
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"])
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"])
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"])

    return temp_df


def stock_zt_pool_em(date: str = "20220426") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-涨停股池
    http://quote.eastmoney.com/ztb/detail#type=ztgc
    :return: 涨停股池
    :rtype: pandas.DataFrame
    """
    url = "http://push2ex.eastmoney.com/getTopicZTPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "10000",
        "sort": "fbt:asc",
        "date": date,
        "_": "1621590489736",
    }
    ses = request_session()
    r = ses.get(url, params=params)
    data_json = r.json()
    if data_json["data"] is None:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "换手率",
        "连板数",
        "首次封板时间",
        "最后封板时间",
        "封板资金",
        "炸板次数",
        "所属行业",
        "涨停统计",
    ]
    temp_df["涨停统计"] = (
            temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
            + "/"
            + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "成交额",
            "流通市值",
            "总市值",
            "换手率",
            "封板资金",
            "首次封板时间",
            "最后封板时间",
            "炸板次数",
            "涨停统计",
            "连板数",
            "所属行业",
        ]
    ]
    temp_df["首次封板时间"] = temp_df["首次封板时间"].astype(str).str.zfill(6)
    temp_df["最后封板时间"] = temp_df["最后封板时间"].astype(str).str.zfill(6)
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"])
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"])
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"])
    temp_df["流通市值"] = pd.to_numeric(temp_df["流通市值"])
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"])
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"])
    temp_df["封板资金"] = pd.to_numeric(temp_df["封板资金"])
    temp_df["炸板次数"] = pd.to_numeric(temp_df["炸板次数"])
    temp_df["连板数"] = pd.to_numeric(temp_df["连板数"])

    return temp_df


def stock_zh_a_spot_em() -> pd.DataFrame:
    """
        东方财富网-沪深京 A 股-实时行情
        https://quote.eastmoney.com/center/gridlist.html#hs_a_board
        :return: 实时行情
        :rtype: pandas.DataFrame
        """
    fields_dict = {
        'f2': '最新价', 'f3': '涨跌幅', 'f4': '涨跌额', 'f5': '成交量', 'f6': '成交额',
        'f7': '振幅', 'f8': '换手率', 'f9': '市盈率动', 'f10': '量比', 'f11': '5分钟涨跌',
        'f12': '代码', 'f14': '名称', 'f15': '最高', 'f16': '最低', 'f17': '今开', 'f18': '昨收',
        'f20': '总市值', 'f21': '流通市值', 'f22': '涨速', 'f23': '市净率', 'f24': '60日涨跌幅',
        'f25': '年初至今涨跌幅', 'f26': '上市时间',
        'f37': '加权净资产收益率', 'f38': '总股本', 'f39': '已流通股份', 'f40': '营业收入', 'f41': '营业收入同比增长',
        'f45': '归属净利润', 'f46': '归属净利润同比增长', 'f48': '每股未分配利润', 'f49': '毛利率',
        'f57': '资产负债率', 'f61': '每股公积金', 'f100': '所处行业', 'f112': '每股收益', 'f113': '每股净资产',
        'f114': '市盈率静', 'f115': '市盈率TTM', 'f221': '报告期'
    }
    url = "http://82.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "50000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80,m:1 t:2,m:1 t:23,m:0 t:81 s:2048",
        "fields": ",".join(fields_dict.keys()),
        "_": "1623833739532",
    }
    ses = request_session()
    r = ses.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.columns = fields_dict.values()

    for col in temp_df.columns:
        if col in ['名称', '代码', '所处行业']:
            continue
        elif col in ['上市时间', '报告期']:
            temp_df[col] = pd.to_datetime(temp_df[col], format='%Y%m%d', errors="coerce")
        else:
            temp_df[col] = pd.to_numeric(temp_df[col], errors="coerce")

    # 数字开头的列名在dolphindb 查询的时候会有问题, 此处增加字母开头
    temp_df = temp_df[
        [
            "代码", "名称",
            "最新价", "涨跌幅", "涨跌额", "成交量", "成交额", "振幅",
            "换手率", "量比", "今开", "最高", "最低", "昨收",
            "涨速", "5分钟涨跌", "60日涨跌幅", "年初至今涨跌幅",
            "市盈率动", "市盈率TTM", "市盈率静", "市净率",
            "每股收益", "每股净资产", "每股公积金", "每股未分配利润", "加权净资产收益率", "毛利率", "资产负债率",
            "营业收入", "营业收入同比增长", "归属净利润", "归属净利润同比增长",
            "报告期", "总股本", "已流通股份", "总市值", "流通市值",
            "所处行业", "上市时间"
        ]
    ].rename(columns={'5分钟涨跌': 'A5分钟涨跌', '60日涨跌幅': 'A60日涨跌幅'})

    return temp_df


if __name__ == '__main__':
    df = stock_zh_a_spot_em()
    df = stock_zt_pool_em()
    df = stock_zh_a_hist()
    print(1)



dfs_header = "dfs://"  # 不能有 sub 文件夹


class DfsDbPath:
    trade_calender = dfs_header + "trade_calender"

    index_daily_code = dfs_header + "index_daily_code"

    stock_basic = dfs_header + "stock_basic"
    stock_index = dfs_header + "stock_index"
    stock_daily_code = dfs_header + "stock_daily_code"
    stock_daily = dfs_header + "stock_daily"  # 资金流

    bond_basic = dfs_header + "bond_basic"
    bond_index = dfs_header + "bond_index"
    bond_daily_code = dfs_header + "bond_daily_code"

    crypto_lob = dfs_header + "crypto_lob"


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 7:08
# @Author   : Fangyang
# @Software : PyCharm

import datetime

from typing import Union, Dict

from gulf.dolphindb.base import Dolphindb
from gulf.dolphindb.const import Engine
from gulf.dolphindb.db_path import DfsDbPath


class CryptoDB(Dolphindb):
    # 用于 数据库分区 和 确定数据下载范围
    start_year = 2022
    end_year = 2030

    def __init__(
            self, host: Union[str, None] = None, port: int = 8848,
            username: str = 'admin', password: str = '123456',
            engine: Engine = Engine.TSDB, clear_db: bool = False,
            enable_async: bool = False
    ):
        super().__init__(
            host=host, port=port, username=username, password=password,
            engine=engine, enable_async=enable_async
        )

        self.start_datetime = datetime.datetime(year=self.start_year, month=1, day=1)

        self.init_db(clear=clear_db)

    @property
    def db_path_init_script_dict(self) -> Dict[str, str]:
        return {
            DfsDbPath.crypto_lob: self.init_btc_lob_db_script,
        }

    @property
    def init_btc_lob_db_script(self):
        return f"""
        db_code_hash = database("", HASH,[SYMBOL,6])
        db_date_value = database(, VALUE, {self.start_year}.01.01..{self.end_year}.01.01)
        db=database('{DfsDbPath.crypto_lob}', COMPO, [db_date_value, db_code_hash], engine=`{self.engine.value})
        """


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:26
# @Author   : Fangyang
# @Software : PyCharm

from gulf.utils.stock import MARKET_BJ, MARKET_SH, MARKET_SZ, get_stock_market

JUEJIN_MARKET_MAP = {
    MARKET_SH: "SHSE",
    MARKET_SZ: "SZSE",
    MARKET_BJ: "BJSE"
}


def trans_to_juejin_code(code):
    return f"{JUEJIN_MARKET_MAP[get_stock_market(code)]}.{code}"


if __name__ == '__main__':
    pass



from gulf.akshare.stock.all_basic import get_all_stocks_df
from gulf.akshare.stock.history import get_stock_hist_df, update_res_dict_thread
from gulf.akshare.stock.analyst_rating import get_stock_analyst_rating



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:58
# @Author   : Fangyang
# @Software : PyCharm

import threading
import time
import pandas as pd

from typing import Dict
from tqdm import tqdm
from loguru import logger

from gulf.akshare.const import AKSHARE_CRAWL_CONCURRENT_LIMIT, AKSHARE_CRAWL_STOP_INTERVEL
from gulf.akshare.stock.migrate_func import stock_zh_a_hist
from gulf.utils.stock import get_stock_ch_market


def get_stock_hist_df(
        res_dict: Dict[str, pd.DataFrame],
        code: str = '000001', name: str = "平安银行",
        start_date: str = "19700101",
        end_date: str = "20500101",
) -> pd.DataFrame:
    """
    和 tushare pro 的数据相比:
    1. 多了 换手率, 但是和 tushare 每日 indicator 中有 turnover, merge的时候补上了
    2. name 不为 None 就多了 名称 ;
    3. 少了复权因子, 交易状态, 平均价格

    这里爬虫获取的 振幅 = (当日最高价 - 当日最低价) / 昨日收盘价 * 100%
    tushare pro 的 change 叫 涨跌 是 (今日close - 昨日close)

    :param code: 000001
    :param name:
    :param start_date:
    :param end_date:
    :param res_dict:
    :return:
    """
    hfq_df_columns_dict = {
        '日期': "trade_date",
        '开盘': "adj_open", '收盘': "adj_close", '最高': "adj_high", '最低': "adj_low",
        '成交量': "volume", '成交额': "amount",
        '涨跌幅': "pct_chg",
        # 下面这俩字段在写数据库时被过滤掉了
        '涨跌额': "pct_amount", '换手率': "turnover",
        '振幅': 'amp'
    }
    try:
        hfq_df = stock_zh_a_hist(
            symbol=code, period="daily",
            start_date=start_date, end_date=end_date,
            adjust="hfq"
        )

        if hfq_df.empty:
            logger.error(f"[stock_zh_a_hist]{code} {name} hfq dataframe is empty. Maybe limited.")
            return pd.DataFrame()

        hfq_df = hfq_df[hfq_df_columns_dict.keys()].rename(columns=hfq_df_columns_dict)

        nfq_df_columns_dict = {'开盘': "open", '收盘': "close", '最高': "high", '最低': "low"}
        nfq_df = stock_zh_a_hist(
            symbol=code, period="daily",
            start_date=start_date, end_date=end_date,
            adjust=""
        )[nfq_df_columns_dict.keys()].rename(
            columns=nfq_df_columns_dict
        )
    except Exception as e:
        logger.error(f'[stock_zh_a_hist]{code} {name}. {e}')
        return pd.DataFrame()

    df = pd.concat([hfq_df, nfq_df], axis=1)

    if name is not None:
        df['name'] = name

    res_dict[code] = df

    return df


def update_res_dict_thread(df: pd.DataFrame, start_date_str: str, end_date_str: str, res_dict: dict):
    start_time = time.perf_counter()
    t_list = []
    for idx, item in tqdm(df.iterrows(), desc="Get stock hist df from akshare"):
        code, name = item[0], item[1]

        ch_market = get_stock_ch_market(code)
        if ch_market not in ['沪A股主板', '深A股主板', '中小板', '创业板']:
            logger.info(f'{code} {name} 是 {ch_market} 股票, 跳过')
            continue

        t = threading.Thread(target=get_stock_hist_df, args=(res_dict, code, name, start_date_str, end_date_str))
        t.start()
        t_list.append(t)

        if len(t_list) > AKSHARE_CRAWL_CONCURRENT_LIMIT:
            [t.join() for t in t_list]
            t_list = []
            time.sleep(AKSHARE_CRAWL_STOP_INTERVEL)

    if t_list:
        [t.join() for t in t_list]

    logger.success(f"Items: {len(res_dict)}/{len(df)}. Threading cost: {time.perf_counter() - start_time:.2f}s")


if __name__ == '__main__':
    from gulf.akshare.stock.all_basic import get_all_stocks_df

    stock_zh_a_spot_em_df = get_all_stocks_df()
    res_dict = dict()

    # 单日数据下载12分钟, 全历史数据下载40多分钟
    update_res_dict_thread(
        df=stock_zh_a_spot_em_df[['代码', '名称']],
        start_date_str="20220902",
        end_date_str="20220902",
        res_dict=res_dict
    )

    res_list = []
    for k, v in tqdm(res_dict.items(), desc=f"Concating res_df"):
        # copy: 00:01:24
        # no copy : 00:02:35
        # list append then concat once: 00:00:00
        res_list.append(v)
    res_df = pd.concat(res_list)
    print(1)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Datetime : 2023/1/26 0:37
# @Author   : Fangyang
# @Software : PyCharm

import pandas as pd


def trans_ch_col_name(func):
    def inner(*args, **kwargs) -> pd.DataFrame:
        df: pd.DataFrame = func(*args, **kwargs)
        df.rename(columns={
            '板块名称': "jj_code",
            "日期": "trade_date",
            "日期时间": "trade_date",
            '开盘': "open", '收盘': "close", '最高': 'high', '最低': 'low',
            '涨跌幅': "pct_chg", '涨跌额': "pct_amount",
            '成交量': "vol", '成交额': "amount", '振幅': 'hl_chg', '换手率': "turnover_rate",
        }, inplace=True)
        df['trade_date'] = pd.to_datetime(df['trade_date'])
        return df

    return inner


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Datetime : 2023/3/5 21:06
# @Author   : Fangyang
# @Software : PyCharm

import pandas as pd


def test_handler(els):
    global test_df
    test_df = pd.concat([test_df, pd.Series([els])])
    print(test_df.shape)
    print(els)


if __name__ == '__main__':
    from gulf.dolphindb.crypto import CryptoDB
    from threading import Event
    from stream_table_test_pub import table_name
    from gulf.dolphindb.tables.stream.stream_table import StreamTable

    db = CryptoDB()

    db.session.enableStreaming()

    if db.session.run(StreamTable.get_all_persist_table()) is not None:
        test_df = db.session.run(f"select * from {table_name}")
        db.session.subscribe(
            db.host, 8848, handler=test_handler, tableName=table_name,
            # offset=100
        )
        Event().wait()
    else:
        print("None persist_table or dolphindb session is ASYNC=True.")

    # db.session.run(t.drop_table())



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:21
# @Author   : Fangyang
# @Software : PyCharm

# TODO
# stock 100,5 -> 4328/5529, 240s
# stock 50,5 -> 4328/5529, 619s
AKSHARE_CRAWL_CONCURRENT_LIMIT = 50
AKSHARE_CRAWL_STOP_INTERVEL = 30  # seconds

if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:29
# @Author   : Fangyang
# @Software : PyCharm

from gulf.dolphindb.const import DolType
from gulf.dolphindb.tables.partition.schema import new_schema, Schema


def stock_nfq_daily_table_schema() -> Schema:
    cols_type_dict = {'jj_code': DolType.SYMBOL, }

    # tushare daily 数据不全, 这里使用akshare数据
    daily_cols_type_dict = {
        'trade_date': DolType.NANOTIMESTAMP,
        'open': DolType.DOUBLE, 'high': DolType.DOUBLE,
        'low': DolType.DOUBLE, 'close': DolType.DOUBLE,
        'amount': DolType.DOUBLE, 'volume': DolType.DOUBLE,
    }

    custom_dict = {'name': DolType.SYMBOL, 'industry': DolType.SYMBOL}

    cols_type_dict.update(daily_cols_type_dict)
    cols_type_dict.update(custom_dict)

    columns_list = [
        list(daily_cols_type_dict.keys()),
        list(custom_dict.keys()),
    ]

    return new_schema(cols_type_dict, columns_list)


def stock_daily_table_schema() -> Schema:
    cols_type_dict = {'jj_code': DolType.SYMBOL, }

    # tushare daily 数据不全, 这里使用akshare数据
    daily_cols_type_dict = {
        'ts_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
        # 'pre_close': DolType.DOUBLE,
        'open': DolType.DOUBLE, 'high': DolType.DOUBLE,
        'low': DolType.DOUBLE, 'close': DolType.DOUBLE,
        # 'change': DolType.DOUBLE,
        'pct_chg': DolType.DOUBLE, 'volume': DolType.DOUBLE,
        'amount': DolType.DOUBLE,
        # 'adj_pre_close': DolType.DOUBLE,
        'adj_open': DolType.DOUBLE, 'adj_high': DolType.DOUBLE, 'adj_low': DolType.DOUBLE,
        'adj_close': DolType.DOUBLE,
        # 'adj_factor': DolType.DOUBLE, 'avg_price': DolType.DOUBLE,
        # 'trade_status': DolType.SYMBOL,
    }

    # 由于akshare的 pe pb ps 数据质量没有tushare好, 所以这里都用tushare数据
    daily_basic_ts_cols_type_dict = {
        'ts_code': DolType.SYMBOL, 'trade_date': DolType.NANOTIMESTAMP,
        # 'close': DolType.DOUBLE,
        'turnover_rate': DolType.DOUBLE,
        'turnover_rate_f': DolType.DOUBLE,
        'volume_ratio': DolType.DOUBLE,
        'pe': DolType.DOUBLE, 'pe_ttm': DolType.DOUBLE,
        'pb': DolType.DOUBLE,
        'ps': DolType.DOUBLE, 'ps_ttm': DolType.DOUBLE,
        'dv_ratio': DolType.DOUBLE, 'dv_ttm': DolType.DOUBLE,
        'total_share': DolType.DOUBLE,
        'float_share': DolType.DOUBLE,
        'free_share': DolType.DOUBLE,
        'total_mv': DolType.DOUBLE, 'circ_mv': DolType.DOUBLE,
        # 'udlimit_status': DolType.INT   # 有 nan, pandas 转 int 报错
    }
    custom_dict = {'name': DolType.SYMBOL, 'industry': DolType.SYMBOL}

    cols_type_dict.update(daily_cols_type_dict)
    cols_type_dict.update(daily_basic_ts_cols_type_dict)
    cols_type_dict.update(custom_dict)

    columns_list = [
        list(daily_cols_type_dict.keys()),
        list(daily_basic_ts_cols_type_dict.keys()),
        list(custom_dict.keys()),
    ]

    return new_schema(cols_type_dict, columns_list)


def stock_moneyflow_daily_table_schema() -> Schema:
    cols_type_dict = {}

    # tushare daily 数据不全, 这里使用akshare数据
    daily_cols_type_dict = {
        'in_amount_main': DolType.DOUBLE, 'in_ratio_main': DolType.DOUBLE,
        'in_amount_large': DolType.DOUBLE, 'in_ratio_large': DolType.DOUBLE,
        'in_amount_big': DolType.DOUBLE, 'in_ratio_big': DolType.DOUBLE,
        'in_amount_mid': DolType.DOUBLE, 'in_ratio_mid': DolType.DOUBLE,
        'in_amount_small': DolType.DOUBLE, 'in_ratio_small': DolType.DOUBLE,
        'jj_code': DolType.SYMBOL, 'name': DolType.SYMBOL, 'industry': DolType.SYMBOL,
        'trade_date': DolType.NANOTIMESTAMP,
    }

    cols_type_dict.update(daily_cols_type_dict)

    columns_list = [
        list(daily_cols_type_dict.keys()),
    ]

    return new_schema(cols_type_dict, columns_list)


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:39
# @Author   : Fangyang
# @Software : PyCharm

import pandas as pd

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables.dimension.table import DimensionTable


class StockBasicTable(DimensionTable):
    name = "stock_basic_table"
    db_path = DfsDbPath.stock_basic
    db_name = "stock_basic_db"
    sort_columns = 'jj_code'

    def __init__(self):
        super().__init__()

        from gulf.akshare.stock import get_all_stocks_df

        table_df = get_all_stocks_df()
        # table_df['list_date'] = pd.to_datetime(table_df['list_date'])
        # table_df['delist_date'] = pd.to_datetime(table_df['delist_date'])

        self.df = table_df


class TradeCalenderTable(DimensionTable):
    name = "trade_calender_table"
    db_path = DfsDbPath.trade_calender
    db_name = "trade_calender_db"
    sort_columns = 'index'

    def __init__(self):
        super().__init__()

        import akshare as ak

        df = ak.tool_trade_date_hist_sina().astype('str')
        df['trade_date'] = pd.to_datetime(df['trade_date'])
        df.reset_index(inplace=True)
        self.df = df


class IndexHS300MembersTable(DimensionTable):
    name = "index_hushen300_members_table"
    db_path = DfsDbPath.stock_index
    db_name = "stock_index_db"
    sort_columns = "jj_code"

    def __init__(self):
        """
        ['jj_code', 'ts_code', 'con_ts_code', 'in_date', 'out_date', 'cur_sign']

        名称	        类型	    默认显示	描述
        ts_code 	str	    Y	    ts代码
        con_ts_code	str	    Y	    成份股ts代码
        in_date	    str	    Y	    纳入日期
        out_date	str	    Y	    剔除日期
        cur_sign	float	Y	    最新标志
        """
        super().__init__()

        from avalon.datafeed.xcsc_tushare import get_hushen300_index_members_df, trans_tushare_code_to_juejin_code

        table_df = get_hushen300_index_members_df()
        table_df['jj_code'] = trans_tushare_code_to_juejin_code(table_df['con_ts_code'])
        table_df['in_date'] = pd.to_datetime(table_df['in_date'])
        table_df['out_date'] = pd.to_datetime(table_df['out_date'])
        self.df = table_df


class IndexHS300DailyTable(DimensionTable):
    name = "index_hushen300_members_table"
    db_path = DfsDbPath.stock_index
    db_name = "stock_index_db"
    sort_columns = 'index'

    def __init__(self):
        super().__init__()
        from avalon.datafeed.xcsc_tushare import trans_tushare_code_to_juejin_code
        from avalon.datafeed.xcsc_tushare.stock import get_hushen300_index_daily_df

        df = get_hushen300_index_daily_df()
        df['trade_date'] = pd.to_datetime(df['trade_date'])
        df['jj_code'] = trans_tushare_code_to_juejin_code(df['ts_code'])
        df.reset_index(inplace=True)

        self.df = df


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:29
# @Author   : Fangyang
# @Software : PyCharm


from typing import Union, List, Any

# 市场
MARKET_SZ = 0  # 深市
MARKET_SH = 1  # 沪市
MARKET_BJ = 2  # 北交


def get_stock_markets(symbols=None) -> List[List[Union[Union[int, str], Any]]]:
    results = []

    assert isinstance(symbols, list), 'stock code need list type'

    if isinstance(symbols, list):
        for symbol in symbols:
            results.append([get_stock_market(symbol, string=False), symbol.strip('sh').strip('sz')])

    return results


def get_stock_market(symbol: str = '', string: bool = False) -> Union[int, str]:
    """ 判断股票ID对应的证券市场匹配规则

    ['50', '51', '60', '90', '110'] 为 sh
    ['00', '12'，'13', '18', '15', '16', '18', '20', '30', '39', '115'] 为 sz
    ['5', '6', '9'] 开头的为 sh， 其余为 sz

    :param string: False 返回市场ID，否则市场缩写名称
    :param symbol: 股票ID, 若以 'sz', 'sh' 开头直接返回对应类型，否则使用内置规则判断
    :return 'sh' or 'sz'
    """

    assert isinstance(symbol, str), 'stock code need str type'

    market = 'sh'

    if symbol.startswith(('sh', 'sz', 'SH', 'SZ')):
        market = symbol[:2].lower()

    elif symbol.startswith(('50', '51', '60', '68', '90', '110', '113', '132', '204')):
        market = 'sh'

    elif symbol.startswith(('00', '12', '13', '18', '15', '16', '18', '20', '30', '39', '115', '1318')):
        market = 'sz'

    elif symbol.startswith(('5', '6', '9', '7')):
        market = 'sh'

    elif symbol.startswith(('4', '8')):
        market = 'bj'

    if string is False:

        if market == 'sh':
            market = MARKET_SH

        if market == 'sz':
            market = MARKET_SZ

        if market == 'bj':
            market = MARKET_BJ

    return market


def get_stock_ch_market(code: str):
    if code[:3] in ['600', '601', '602', '603', '605']:
        return '沪A股主板'
    elif code[:3] in ['000', '001', '003']:
        return '深A股主板'
    elif code[:3] == '002':
        return '中小板'
    elif code[:2] == '30':
        return '创业板'
    elif code[:3] == '688':
        return '科创板'
    elif code[0] == '4':
        return '三板'
    elif code[0] == '8':
        return '新三板'
    elif code[:3] in '900':
        return '沪B股'
    elif code[:3] in '200':
        return '深B股'
    elif code[:3] == '730':
        return '新股申购'
    elif code[:3] in ['700', '080']:
        return '配股'


if __name__ == '__main__':
    pass



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:40
# @Author   : Fangyang
# @Software : PyCharm
import pandas as pd

from gulf.dolphindb.db_path import DfsDbPath
from gulf.dolphindb.tables.dimension.table import DimensionTable


class BondBasicTable(DimensionTable):
    name = "bond_basic_table"
    db_path = DfsDbPath.bond_basic
    db_name = "bond_basic_db"
    sort_columns = 'jj_code'

    def __init__(self):
        super().__init__()

        from gulf.akshare.bond import get_bond_basic_df

        table_df, _ = get_bond_basic_df()
        table_df['上市时间'] = pd.to_datetime(table_df['上市时间'])
        table_df['转股起始日'] = pd.to_datetime(table_df['转股起始日'])

        self.df = table_df


class BondRedeemTable(DimensionTable):
    name = "bond_redeem_table"
    db_path = DfsDbPath.bond_basic
    db_name = "bond_basic_db"
    sort_columns = 'jj_code'

    def __init__(self):
        super().__init__()

        from gulf.akshare.bond import get_bond_basic_df

        _, redeem_table_df = get_bond_basic_df()
        redeem_table_df['转股起始日'] = pd.to_datetime(redeem_table_df['转股起始日'])

        self.df = redeem_table_df


class BondDelistTable(DimensionTable):
    name = "bond_delist_table"
    db_path = DfsDbPath.bond_basic
    db_name = 'bond_basic_db'
    sort_columns = 'jj_code'

    def __init__(self):
        """
        117*** 是 EB可交换债, 正股代码 和 名称 为 None
        """
        super(BondDelistTable, self).__init__()

        from avalon.datafeed.xcsc_tushare import get_delist_bond_df

        table_df = get_delist_bond_df()
        table_df['trade_dt_last'] = pd.to_datetime(table_df['trade_dt_last'])
        self.df = table_df


class BondIndexTable(DimensionTable):
    name = "bond_index_table"
    db_path = DfsDbPath.bond_index
    db_name = 'bond_index_db'
    sort_columns = 'index'

    def __init__(self):
        super(BondIndexTable, self).__init__()

        from gulf.akshare.bond import get_bond_index_daily

        table_df = get_bond_index_daily()
        table_df['trade_date'] = pd.to_datetime(table_df['trade_date'])
        table_df.reset_index(inplace=True)
        self.df = table_df


if __name__ == '__main__':
    pass



import threading
import time

import akshare as ak
import pandas as pd
from tqdm import tqdm
from datetime import date

from gulf.akshare.const import AKSHARE_CRAWL_STOP_INTERVEL, AKSHARE_CRAWL_CONCURRENT_LIMIT
from gulf.akshare.stock.index_decorator import trans_ch_col_name


def update_index_industry_em_df_thread(symbol, start_date, end_date, res_dict):
    if start_date is not None:
        # 数据带复权, 可以指定起始结束日期
        df = ak.stock_board_industry_hist_em(
            symbol=symbol,
            start_date=start_date,
            end_date=date.today().strftime("%Y%m%d") if end_date is None else end_date,
            period="日k",
            adjust="hfq"
        )
    else:
        # 返回从开始到现在的全部数据, 未复权, 复权对于行业指数有用??
        df = ak.stock_board_industry_hist_min_em(symbol=symbol, period="101")

    df['板块名称'] = symbol
    res_dict[symbol] = df


@trans_ch_col_name
def get_stock_index_industry_em_daily_df(start_date=None, end_date=None) -> pd.DataFrame:
    ind_name = ak.stock_board_industry_name_em()
    t_list = []
    res_dict = dict()
    for bk_name in tqdm(ind_name['板块名称'].values):
        # update_index_industry_em_df_thread(bk_name, start_date, end_date, res_dict)
        t = threading.Thread(
            target=update_index_industry_em_df_thread,
            args=(bk_name, start_date, end_date, res_dict)
        )
        t.start()
        t_list.append(t)

        if len(t_list) > AKSHARE_CRAWL_CONCURRENT_LIMIT:
            [t.join() for t in t_list]
            t_list = []

            print(f" In case of ban ip, wait {AKSHARE_CRAWL_STOP_INTERVEL}s ...")
            time.sleep(AKSHARE_CRAWL_STOP_INTERVEL)

    [t.join() for t in t_list]
    return pd.concat(list(res_dict.values()))


if __name__ == '__main__':
    file_name = "stock_index_industry_em_daily_df.pkl"
    # stock_index_industry_em_daily_df = get_stock_index_industry_em_daily_df(
    #     start_date="20230101"
    # )
    # stock_index_industry_em_daily_df = get_stock_index_industry_em_daily_df(
    #     start_date="20230113", end_date="20230113"
    # )
    stock_index_industry_em_daily_df = get_stock_index_industry_em_daily_df()
    stock_index_industry_em_daily_df.to_pickle(file_name)
    stock_index_industry_em_daily_df = pd.read_pickle(file_name)
    print(1)

    # ind_name = ak.stock_board_industry_name_em()
    # ths_ind_name1 = ak.stock_board_industry_name_ths()
    # ths_ind_name2 = ak.stock_board_industry_summary_ths()
    # print(1)

    # stock_board_change_em_df = ak.stock_board_change_em()
    # print(stock_board_change_em_df)
